Package: RGENERATEPREC
Maintainer: Emanuele Cordano <emanuele.cordano@gmail.com>
License: GPL (>= 2)
Title: Tools to Generate Daily-Precipitation Time Series
Type: Package
Authors@R: c(person(given = "Emanuele",
                        family = "Cordano",
                        role = c("aut", "cre","ctb"),
                        email = "emanuele.cordano@gmail.com",comment=c(ORCID="0000-0002-3508-5898")))
Description: The method 'generate()' is extended for spatial multi-site
    stochastic generation of daily precipitation. It generates precipitation
    occurrence in several sites using logit regression (Generalized Linear
    Models) and the approach by D.S. Wilks (1998) <doi:10.1016/S0022-1694(98)00186-3> . 
Version: 1.3.2
Repository: CRAN
Date: 2025-07-30
Depends: R (>= 3.5.0), copula, RGENERATE, blockmatrix, Matrix, stringr
Imports: RMAWGEN,lubridate
Suggests: knitr,rmarkdown,mapview,sf,lmom,ggplot2,reshape2,RefManageR
VignetteBuilder: knitr
URL: https://ecor.github.io/RGENERATEPREC/
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-07-30 14:16:53 UTC; ecor
Author: Emanuele Cordano [aut, cre, ctb] (ORCID:
    <https://orcid.org/0000-0002-3508-5898>)
Date/Publication: 2025-07-30 16:10:02 UTC
Built: R 4.6.0; ; 2026-04-26 09:58:14 UTC; unix
