# Generated by roxygen2: do not edit by hand

S3method(generate,CCGammaObjectListPerEachMonth)
S3method(generate,PrecipitationAmountModel)
S3method(generate,PrecipitationOccurrenceModel)
S3method(generate,PrecipitationOccurrenceMultiSiteModel)
S3method(predict,PrecipitationAmountModel)
S3method(predict,PrecipitationOccurrenceModel)
S3method(predict,PrecipitationOccurrenceMultiSiteModel)
export(CCGamma)
export(CCGammaToBlockmatrix)
export(PrecipitationAmountModel)
export(PrecipitationOccurrenceModel)
export(PrecipitationOccurrenceMultiSiteModel)
export(dw.spell)
export(nwetdays)
export(omega)
export(omega_inv)
export(omega_root)
import(Matrix)
import(blockmatrix)
import(copula)
import(stringr)
importFrom(RGENERATE,generate)
importFrom(RMAWGEN,adddate)
importFrom(RMAWGEN,continuity_ratio)
importFrom(RMAWGEN,normalizeGaussian_severalstations)
importFrom(lubridate,day)
importFrom(lubridate,month)
importFrom(lubridate,year)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,residuals)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,uniroot)
importFrom(utils,str)
